@@ -4,21 +4,17 @@
 import collections
 import copy
 import itertools
-import re
-import sys
 import logging
 import pkg_resources
 
 from c8connector import (
     C8Connector, ConfigProperty, Sample,
     ConfigAttributeType, SchemaAttributeType, SchemaAttribute)
-
 from c8connector import Schema as C8Schema
 
 import singer
 import singer.metrics as metrics
 import singer.schema
-import snowflake.connector
 from singer import metadata
 from singer import utils
 from singer.catalog import Catalog, CatalogEntry
@@ -33,7 +29,7 @@ from macrometa_source_snowflake.connection import SnowflakeConnection
 LOGGER = singer.get_logger('macrometa_source_snowflake')
 
 # Max number of rows that a SHOW SCHEMAS|TABLES|COLUMNS can return.
-# If more than this number of rows returned then tap-snowflake will raise TooManyRecordsException
+# If more than this number of rows returned then macrometa-source-snowflake will raise TooManyRecordsException
 SHOW_COMMAND_MAX_ROWS = 9999
 
 
@@ -225,6 +221,11 @@ def discover_catalog(snowflake_conn, config):
         md_map = metadata.write(md_map, (), 'database-name', table_catalog)
         md_map = metadata.write(md_map, (), 'schema-name', table_schema)
 
+        # Unique primary key provided by user
+        if config.get('primary_key') and any(column.column_name == config.get('primary_key') for column in cols):
+            md_map = metadata.write(
+                md_map, (), 'table-key-properties', [config.get('primary_key')])
+
         if (
                 table_catalog in table_info and
                 table_schema in table_info[table_catalog] and
@@ -558,7 +559,6 @@ class SnowflakeSourceConnector(C8Connector):
         except Exception as e:
             raise e
 
-    # TODO: Need to add handling of reserved keys
     def samples(self, integration: dict) -> list[Sample]:
         """Fetch sample data using the provided configurations."""
         config = self.get_config(integration)
@@ -566,28 +566,29 @@ class SnowflakeSourceConnector(C8Connector):
             snowflake_conn = SnowflakeConnection(config)
             catalog = discover_catalog(snowflake_conn, config)
             results = []
-            
+
             for stream in catalog.streams:
                 s_attribs = []
                 s_schema = stream.schema
-                LOGGER.info('DEBUG SAMPLE 3: %s', s_schema)
                 data = fetch_samples(config, stream)[:10]
-                LOGGER.info('DEBUG SAMPLE 4: %s', data)
+
                 # Appending _ to keys for preserving values of reserved keys in source data
                 reserved_keys = ['_key', '_id', '_rev']
                 metadata_obj = stream.metadata[0]
-                if metadata_obj.metadata.get('table-key-properties'):
-                    key_properties = metadata_obj.metadata.get('table-key-properties')
+                if metadata_obj['metadata'].get('table-key-properties'):
+                    key_properties = metadata_obj['metadata'].get(
+                        'table-key-properties')
                     if key_properties[0] == '_key':
-                            reserved_keys.remove('_key')
+                        reserved_keys.remove('_key')
                 properties = s_schema.properties
-                modified_properties = modify_reserved_keys(properties, reserved_keys)
+                modified_properties = modify_reserved_keys(
+                    properties, reserved_keys)
                 s_schema.properties = modified_properties
+
                 for k, v in s_schema.properties.items():
                     t = v.type[-1]
                     s_attribs.append(SchemaAttribute(
                         k, self.get_attribute_type(t)))
-                LOGGER.info('DEBUG SAMPLE 6: %s', str(s_attribs))
                 schema = C8Schema(stream.stream, s_attribs)
                 results.append(Sample(
                     schema=schema,
@@ -598,37 +599,37 @@ class SnowflakeSourceConnector(C8Connector):
             raise e
         return results
 
-    # TODO: Need to add handling of reserved keys
     def schemas(self, integration: dict) -> list[C8Schema]:
         """Get supported schemas using the given configurations."""
         config = self.get_config(integration)
         try:
             snowflake_conn = SnowflakeConnection(config)
-            LOGGER.info('DEBUG 1: %s', config)
             catalog = discover_catalog(snowflake_conn, config)
-            LOGGER.info('DEBUG 2: %s', catalog)
             results = []
             for stream in catalog.streams:
                 s_attribs = []
                 s_schema = stream.schema
-                LOGGER.info('DEBUG 3: %s', s_schema)
+
                 # Appending _ to keys for preserving values of reserved keys in source data
                 reserved_keys = ['_key', '_id', '_rev']
                 metadata_obj = stream.metadata[0]
-                if metadata_obj.metadata.get('table-key-properties'):
-                    key_properties = metadata_obj.metadata.get('table-key-properties')
+                if metadata_obj['metadata'].get('table-key-properties'):
+                    key_properties = metadata_obj['metadata'].get(
+                        'table-key-properties')
                     if key_properties[0] == '_key':
-                            reserved_keys.remove('_key')
+                        reserved_keys.remove('_key')
                 properties = s_schema.properties
-                modified_properties = modify_reserved_keys(properties, reserved_keys)
+                modified_properties = modify_reserved_keys(
+                    properties, reserved_keys)
                 s_schema.properties = modified_properties
+
                 for k, v in s_schema.properties.items():
                     t = v.type[-1]
                     s_attribs.append(SchemaAttribute(
                         k, self.get_attribute_type(t)))
-                LOGGER.info('DEBUG 4: %s', s_attribs)
                 results.append(C8Schema(stream.stream, s_attribs))
         except Exception as e:
+            LOGGER.info('Exception raised: %s', e)
             raise e
         return results
 
@@ -653,39 +654,47 @@ class SnowflakeSourceConnector(C8Connector):
         """Get configuration parameters for the connector."""
         return [
             ConfigProperty('account', 'Account', ConfigAttributeType.STRING, True, False,
-                           description='Snowflake account',
-                           placeholder_value='snowflake_account'),
+                           description='Snowflake account.',
+                           placeholder_value='my_account'),
             ConfigProperty('dbname', 'Database Name', ConfigAttributeType.STRING, True, False,
-                           description='Snowflake database name',
-                           placeholder_value='snowflake'),
+                           description='Snowflake database name.',
+                           placeholder_value='SNOWFLAKE'),
             ConfigProperty('user', 'Username', ConfigAttributeType.STRING, True, False,
-                           description='Snowflake user',
-                           placeholder_value='user'),
+                           description='Snowflake username.',
+                           placeholder_value='my_user'),
             ConfigProperty('password', 'Password', ConfigAttributeType.STRING, True, False,
-                           description='Snowflake password',
-                           placeholder_value='password'),
+                           description='Snowflake password.',
+                           placeholder_value='my_password'),
             ConfigProperty('warehouse', 'Warehouse', ConfigAttributeType.STRING, True, False,
-                           description='Snowflake virtual warehouse name',
-                           placeholder_value='my_schema'),
+                           description='Snowflake virtual warehouse name.',
+                           placeholder_value='my_warehouse'),
             ConfigProperty('tables', 'Tables', ConfigAttributeType.STRING, True, False,
-                           description='A comma separated list of the table names that you want to sync. The table names should be fully qualified including the db and schema name. For example, syncing table1 and table2 would look like `db.schema.table1,db.schema.table2`',
-                           placeholder_value='my_table'),
+                           description='A comma separated list of the table names that you want to sync.'
+                                       ' The table names should be fully qualified including the db and schema name.'
+                                       ' For example, syncing table1 and table2 would look like'
+                                       ' `db.schema.table1,db.schema.table2`.',
+                           placeholder_value='my_db.my_schema.my_table'),
             ConfigProperty('replication_method', 'Replication Method',
                            ConfigAttributeType.STRING, True, False,
                            description='Choose from INCREMENTAL, FULL_TABLE',
                            default_value='FULL_TABLE'),
+            ConfigProperty('primary_key', 'Unique Primary Key (to be used as _key for the collection)', ConfigAttributeType.STRING, False, False,
+                           description='A unique primary key from the snowflake table which needs be used as _key for the collection.'
+                                       ' If the columns does not have any column with unique values then do not specify anything here,'
+                                       ' _key for the collection will be autogenerated in this case.',
+                           placeholder_value='my_db.my_schema.my_table'),
             ConfigProperty('role', 'Role',
                            ConfigAttributeType.STRING, False, False,
-                           description='Snowflake role to use',
+                           description='Snowflake role to use.',
                            placeholder_value='my_role'),
-            ConfigProperty('private_key_path', 'Private Key Path', ConfigAttributeType.STRING, False, False,
-                           description='A path to your private key used for authenticating using Key Pair authentication instead of user/pass',
-                           placeholder_value='private_key_path'),
+            ConfigProperty('private_key_path', 'Private Key', ConfigAttributeType.STRING, False, False,
+                           description='A private key used for authenticating using Key Pair authentication instead of user/pass.',
+                           placeholder_value='my_private_key'),
             ConfigProperty('private_key_passphrase', 'Private Key Passphrase', ConfigAttributeType.STRING, False, False,
-                           description='The private key passphrase used for authenticating using Key Pair authentication instead of user/pass',
-                           placeholder_value='private_key_passphrase'),
+                           description='The private key passphrase used for authenticating using Key Pair authentication instead of user/pass.',
+                           placeholder_value='my_private_key_passphrase'),
             ConfigProperty('insecure_mode', 'Insecure Mode', ConfigAttributeType.BOOLEAN, False, False,
-                           description='Use insecure mode to avoid "Failed to get OCSP response" warnings',
+                           description='Use insecure mode to avoid "Failed to get OCSP response" warnings.',
                            default_value=False)
         ]
 
@@ -707,10 +716,7 @@ class SnowflakeSourceConnector(C8Connector):
                 'warehouse': integration['warehouse'],
                 'tables': integration['tables'],
                 # Optional config keys
-
-                # TODO: Check if this is necessery
-                'tap_id': integration.get('tap_id'),
-
+                'primary_key': integration.get('primary_key'),
                 'role': integration.get('role'),
                 'private_key_path': integration.get('private_key_path'),
                 'private_key_passphrase': integration.get('private_key_passphrase'),
